@@ -40,9 +40,97 @@ def simple_ndim_visualizer(data_mat, pos_dims, spec_dims, spec_xdim=None, pos_xd
     verbose : bool, optional
         Whether or not to print log statements
     """
+    label_fontsize = 12
+    subtitle_fontsize = 13
+
+    # ############################# VALIDATING ALL INPUTS ##############################################################
+
+    for parm, parm_name in zip([pos_dims, spec_dims], ['pos_dims', 'spec_dims']):
+        if not isinstance(parm, (list, tuple)):
+            raise TypeError('Expected {} to be of type: Iterable - example list or tuple'.format(parm_name))
+        for item in parm:
+            if not isinstance(item, Dimension):
+                raise TypeError('Expected items in {} to be of type: Dimension'.format(parm_name))
+        if len(parm) > 2:
+            raise NotImplementedError('Currently not able to handle more than 2 position or spectroscopic dimensions.'
+                                      ' {} contains {} dimensions'.format(parm_name, len(parm)))
+        elif len(parm) < 1:
+            raise ValueError('{} contains too few ({}) dimensions'.format(parm_name, len(parm)))
+
+    if len(pos_dims) + len(spec_dims) != data_mat.ndim:
+        raise ValueError('Lengths of pos_dims: {} and spec_dims: {} not matching with that of the dimensions of '
+                         'data_mat: {}'.format(len(pos_dims), len(spec_dims), data_mat.ndim))
+
+    # now check if the dimension matches with that of the N dimensional dataset
+    for parm, dim_type in zip([pos_dims, spec_dims], ['Position', 'Spectroscopic']):
+        offset = 0
+        if dim_type == 'Spectroscopic':
+            offset += len(pos_dims)
+        for ind, item in enumerate(parm):
+            actual_ind = ind + offset
+            if item.values is None:
+                # Let's take this opportunity to fill in the values:
+                item.values = np.arange(data_mat.shape[actual_ind])
+                if verbose:
+                    print('automatically generated reference {} values for dimension: {}'.format(dim_type, item.name))
+            else:
+                if len(item.values) != data_mat.shape[actual_ind]:
+                    raise ValueError(
+                        '{} dimension {} of size {} in the dataset does not have values of the same length: {}'
+                        '.'.format(dim_type, item.name, data_mat.shape[actual_ind], len(item.values)))
+
+    # Is there anything worth visualizing interactively at all in positions or spectroscopic?
+    pos_size = np.prod([len(item.values) for item in pos_dims])
+    spec_size = np.prod([len(item.values) for item in spec_dims])
+    if pos_size == 1 or spec_size == 1:
+        raise ValueError('Too few position: {} or spectroscopic: {} values to visualize interactively. '
+                         'Consider using alternate visualization approaches'.format(pos_size, spec_size))
+
+    # create a dictionary that will allow lookup of values and units by name:
+    pos_dims_dict = {}
+    for dimension in pos_dims:
+        pos_dims_dict[dimension.name] = dimension
+
+    spec_dims_dict = {}
+    for dimension in spec_dims:
+        spec_dims_dict[dimension.name] = dimension
+
+    if spec_xdim is not None:
+        if not isinstance(spec_xdim, (str, unicode)):
+            raise TypeError('spec_xdim should have been a string')
+        if spec_xdim not in spec_dims_dict.keys():
+            raise KeyError('{} not among the provided spectroscopic dimensions'.format(spec_xdim))
+
+    if pos_xdim is not None:
+        if not isinstance(pos_xdim, (str, unicode)):
+            raise TypeError('spec_xdim should have been a string')
+        if pos_xdim not in pos_dims_dict.keys():
+            raise KeyError('{} not among the provided position dimensions'.format(pos_xdim))
+
+    pos_plot_2d = len(pos_dims) > 1 and pos_xdim is None
+    spec_plot_2d = len(spec_dims) > 1 and spec_xdim is None
+    if verbose:
+        print('Plot 2D: Positions: {}, Spectroscopic: {}'.format(pos_plot_2d, spec_plot_2d))
+
+    if not spec_plot_2d and spec_xdim is None:
+        # Take the largest dimension you can find:
+        max_ind = np.argmax([len(item.values) for item in spec_dims])
+        spec_xdim = spec_dims[max_ind].name
+        if verbose:
+            print('automatically chose X axis for 1D Spectroscopic plot as {}'.format(spec_xdim))
+
+    if not pos_plot_2d and pos_xdim is None:
+        # Take the largest dimension you can find:
+        max_ind = np.argmax([len(item.values) for item in pos_dims])
+        pos_xdim = pos_dims[max_ind].name
+        if verbose:
+            print('automatically chose X axis for 1D Position plot as {}'.format(pos_xdim))
+
     pos_dim_names = [item.name for item in pos_dims]
     spec_dim_names = [item.name for item in spec_dims]
 
+    # ################################## HELPER FUNCTIONS ##############################################################
+
     def check_data_type(data_mat):
         if data_mat.dtype.names is not None:
             return 2, list(data_mat.dtype.names), None
@@ -96,8 +184,8 @@ def simple_ndim_visualizer(data_mat, pos_dims, spec_dims, spec_xdim=None, pos_xd
         return spgram_slicing
 
     def plot_1d(axis, image_mat, dim_name, dim_dict, component_title):
-        axis.set_xlabel(dim_name + ' (' + dim_dict[dim_name].units + ')', fontsize=16)
-        axis.set_ylabel(component_title, fontsize=16)
+        axis.set_xlabel(dim_name + ' (' + dim_dict[dim_name].units + ')', fontsize=label_fontsize)
+        axis.set_ylabel(component_title, fontsize=label_fontsize)
         if image_mat.shape[0] != dim_dict[dim_name].values.size:
             image_mat = image_mat.T
         img_handle = axis.plot(dim_dict[dim_name].values, image_mat)
@@ -105,8 +193,8 @@ def simple_ndim_visualizer(data_mat, pos_dims, spec_dims, spec_xdim=None, pos_xd
             other_dims = list(dim_dict.keys()).copy()
             other_dims.remove(dim_name)
             other_dims = other_dims[0]
-            axis.legend(dim_dict[other_dims].values, fontsize=14)
-        set_tick_font_size(axis, 14)
+            axis.legend(dim_dict[other_dims].values)  # , fontsize=14)
+        # set_tick_font_size(axis, 14)
         return img_handle
 
     def plot_2d(axis, image_mat, clims, dim_list):
@@ -115,8 +203,8 @@ def simple_ndim_visualizer(data_mat, pos_dims, spec_dims, spec_xdim=None, pos_xd
                                                                  dim_list[0].values.shape))
         img, cbar = plot_map(axis, image_mat, aspect='auto', clim=clims,
                              x_vec=dim_list[1].values, y_vec=dim_list[0].values)
-        axis.set_xlabel(dim_list[1].name + ' (' + dim_list[1].units + ')', fontsize=16)
-        axis.set_ylabel(dim_list[0].name + ' (' + dim_list[0].units + ')', fontsize=16)
+        axis.set_xlabel(dim_list[1].name + ' (' + dim_list[1].units + ')', fontsize=label_fontsize)
+        axis.set_ylabel(dim_list[0].name + ' (' + dim_list[0].units + ')', fontsize=label_fontsize)
         return img, cbar
 
     def update_image(axis, img_handle, data_mat, slice_dict, twoD=True):
@@ -131,78 +219,7 @@ def simple_ndim_visualizer(data_mat, pos_dims, spec_dims, spec_xdim=None, pos_xd
                 line_handle.set_ydata(y_vec)
             axis.set_ylim([np.min(y_mat), np.max(y_mat)])
 
-    # ###########################################################################
-
-    for parm, parm_name in zip([pos_dims, spec_dims], ['pos_dims', 'spec_dims']):
-        if not isinstance(parm, (list, tuple)):
-            raise TypeError('Expected {} to be of type: Iterable - example list or tuple'.format(parm_name))
-        for item in parm:
-            if not isinstance(item, Dimension):
-                raise TypeError('Expected items in {} to be of type: Dimension'.format(parm_name))
-        if len(parm) > 2:
-            raise NotImplementedError('Currently not able to handle more than 2 position or spectroscopic dimensions.'
-                                      ' {} contains {} dimensions'.format(parm_name, len(parm)))
-
-    if len(pos_dims) + len(spec_dims) != data_mat.ndim:
-        raise ValueError('Lengths of pos_dims and spec_dims not matching with that of the dimensions of data_mat')
-
-    # now check if the dimension matches with that of the N dimensional dataset
-    for parm, dim_type in zip([pos_dims, spec_dims], ['Position', 'Spectroscopic']):
-        offset = 0
-        if dim_type == 'Spectroscopic':
-            offset += len(pos_dims)
-        for ind, item in enumerate(parm):
-            actual_ind = ind + offset
-            if item.values is None:
-                # Let's take this oppurtunity to fill in the values:
-                item.values = np.arange(data_mat.shape[actual_ind])
-                if verbose:
-                    print('automatically generated reference {} values for dimension: {}'.format(dim_type, item.name))
-            else:
-                if len(item.values) != data_mat.shape[actual_ind]:
-                    raise ValueError(
-                        '{} dimension {} of size {} in the dataset does not have values of the same length: {}'
-                        '.'.format(dim_type, item.name, data_mat.shape[actual_ind], len(item.values)))
-
-    # create a dictionary that will allow lookup of values and units by name:
-    pos_dims_dict = {}
-    for dimension in pos_dims:
-        pos_dims_dict[dimension.name] = dimension
-
-    spec_dims_dict = {}
-    for dimension in spec_dims:
-        spec_dims_dict[dimension.name] = dimension
-
-    if spec_xdim is not None:
-        if not isinstance(spec_xdim, (str, unicode)):
-            raise TypeError('spec_xdim should have been a string')
-        if spec_xdim not in spec_dims_dict.keys():
-            raise KeyError('{} not among the provided spectroscopic dimensions'.format(spec_xdim))
-
-    if pos_xdim is not None:
-        if not isinstance(pos_xdim, (str, unicode)):
-            raise TypeError('spec_xdim should have been a string')
-        if pos_xdim not in pos_dims_dict.keys():
-            raise KeyError('{} not among the provided position dimensions'.format(pos_xdim))
-
-    pos_plot_2d = len(pos_dims) > 1 and pos_xdim is None
-    spec_plot_2d = len(spec_dims) > 1 and spec_xdim is None
-    if verbose:
-        print('Plot 2D: Positions: {}, Spectroscopic: {}'.format(pos_plot_2d, spec_plot_2d))
-
-    if not spec_plot_2d and spec_xdim is None:
-        # Take the largest dimension you can find:
-        max_ind = np.argmax([len(item.values) for item in spec_dims])
-        spec_xdim = spec_dims[max_ind].name
-        if verbose:
-            print('automatically chose X axis for 1D Spectroscopic plot as {}'.format(spec_xdim))
-
-    if not pos_plot_2d and pos_xdim is None:
-        # Take the largest dimension you can find:
-        max_ind = np.argmax([len(item.values) for item in pos_dims])
-        pos_xdim = pos_dims[max_ind].name
-        if verbose:
-            print('automatically chose X axis for 1D Position plot as {}'.format(pos_xdim))
+        # ########################## VISUALIZATION BEGINS ##############################################################
 
     data_type, data_names, data_funcs = check_data_type(data_mat)
 
@@ -241,10 +258,10 @@ def simple_ndim_visualizer(data_mat, pos_dims, spec_dims, spec_xdim=None, pos_xd
         print('Spatial map data shape: {}, Spectrogram data shape: {}'.format(current_spatmap.shape,
                                                                               current_spgram.shape))
 
-    fig, axes = plt.subplots(ncols=2, figsize=(15.5, 7))
+    fig, axes = plt.subplots(ncols=2, figsize=(8, 3.5))
     # axes[0].hold(True)
     spec_titles = get_slice_string(spatmap_slicing, spec_dims)
-    axes[0].set_title('Spatial Map for\n' + component_title + '\n' + spec_titles, fontsize=18)
+    axes[0].set_title('Spatial Map for\n' + component_title + '\n' + spec_titles, fontsize=subtitle_fontsize)
     if pos_plot_2d:
         img_spat, cbar_spat = plot_2d(axes[0], current_spatmap, clims, pos_dims)
         main_vert_line = axes[0].axvline(x=spgram_slicing[pos_dims[1].name], color='k')
@@ -253,7 +270,7 @@ def simple_ndim_visualizer(data_mat, pos_dims, spec_dims, spec_xdim=None, pos_xd
         img_spat = plot_1d(axes[0], current_spatmap, pos_xdim, pos_dims_dict, component_title)
 
     pos_titles = get_slice_string(spgram_slicing, pos_dims)
-    axes[1].set_title('Spectrogram for\n' + component_title + '\n' + pos_titles, fontsize=18)
+    axes[1].set_title('Spectrogram for\n' + component_title + '\n' + pos_titles, fontsize=subtitle_fontsize)
     if spec_plot_2d:
         img_spec, cbar_spec = plot_2d(axes[1], current_spgram, clims, spec_dims)
     else:
@@ -291,12 +308,12 @@ def simple_ndim_visualizer(data_mat, pos_dims, spec_dims, spec_xdim=None, pos_xd
                 if pos_plot_2d:
                     img_spat.set_clim(clims)
                 else:
-                    axes[0].set_ylabel(component_title, fontsize=16)
+                    axes[0].set_ylabel(component_title, fontsize=label_fontsize)
                 update_image(axes[1], img_spec, sub_data, spgram_slicing, twoD=spec_plot_2d)
                 if spec_plot_2d:
                     img_spec.set_clim(clims)
                 else:
-                    axes[1].set_ylabel(component_title, fontsize=16)
+                    axes[1].set_ylabel(component_title, fontsize=label_fontsize)
 
                 spec_titles = get_slice_string(spatmap_slicing, spec_dims)
                 axes[0].set_title('Spatial Map for\n' + component_title + '\n' + spec_titles)
@@ -315,7 +332,8 @@ def simple_ndim_visualizer(data_mat, pos_dims, spec_dims, spec_xdim=None, pos_xd
             spgram_slicing.update(get_spgram_slice_dict(slice_dict=kwargs))
             update_image(axes[1], img_spec, global_vars['sub_data'], spgram_slicing, twoD=spec_plot_2d)
             pos_titles = get_slice_string(spgram_slicing, pos_dims)
-            axes[1].set_title('Spectrogram for\n' + global_vars['component_title'] + '\n' + pos_titles, fontsize=18)
+            axes[1].set_title('Spectrogram for\n' + global_vars['component_title'] + '\n' + pos_titles,
+                              fontsize=subtitle_fontsize)
             if pos_plot_2d:
                 main_vert_line.set_xdata(spgram_slicing[pos_dims[1].name])
                 main_hor_line.set_ydata(spgram_slicing[pos_dims[0].name])
@@ -330,7 +348,8 @@ def simple_ndim_visualizer(data_mat, pos_dims, spec_dims, spec_xdim=None, pos_xd
             spatmap_slicing.update(get_spatmap_slice_dict(slice_dict=kwargs))
             update_image(axes[0], img_spat, global_vars['sub_data'], spatmap_slicing, twoD=pos_plot_2d)
             spec_titles = get_slice_string(spatmap_slicing, spec_dims)
-            axes[0].set_title('Spatial Map for\n' + global_vars['component_title'] + '\n' + spec_titles, fontsize=18)
+            axes[0].set_title('Spatial Map for\n' + global_vars['component_title'] + '\n' + spec_titles,
+                              fontsize=subtitle_fontsize)
 
         slice_dict.update(kwargs)
 
