@@ -12,12 +12,14 @@ import inspect
 import os
 import sys
 from numbers import Number
+import numpy as np
 import h5py
 import matplotlib as mpl
 import matplotlib.pyplot as plt
-import numpy as np
+import matplotlib.ticker as mtick
 from matplotlib.colors import LinearSegmentedColormap
 from mpl_toolkits.axes_grid1 import ImageGrid
+from ..io.dtype_utils import get_exponent
 
 if sys.version_info.major == 3:
     unicode = str
@@ -128,6 +130,35 @@ def set_tick_font_size(axes, font_size):
         __set_axis_tick(axes)
 
 
+def use_scientific_ticks(axis, is_x=True, formatting='%.2e'):
+    """
+    Makes the desired axis use scientific notation for its tick labels. This is applicable only for 1D plots at the
+    moment.
+
+    Parameters
+    ----------
+    axis : matplotlib.pyplot.axis object
+        Axis handle
+    is_x : bool, optional. Default = True
+        If set to true, scientific notation will be applied only to the X axis.
+        If set to False, scientific notation will be applied only to the Y axis.
+    formatting : str / unicode, optional. Default = 2 digits of precision
+        Precision for the tick labels
+    """
+    if not isinstance(axis, mpl.axes.Axes):
+        raise TypeError('axis must be a matplotlib.axes.Axes object')
+    if not isinstance(is_x, bool):
+        raise TypeError('is_x should be a boolean to avoid confusion')
+    if not isinstance(formatting, (str, unicode)):
+        raise TypeError('formatting must be a string')
+
+    if is_x:
+        ax_hand = axis.xaxis
+    else:
+        ax_hand = axis.yaxis
+    ax_hand.set_major_formatter(mtick.FormatStrFormatter(formatting))
+
+
 def make_scalar_mappable(vmin, vmax, cmap=None):
     """
     Creates a scalar mappable object that can be used to create a colorbar for non-image (e.g. - line) plots
@@ -462,6 +493,7 @@ def plot_line_family(axis, x_vec, line_family, line_names=None, label_prefix='',
         quantity by which the lines are offset from each other vertically (useful for spectra)
     show_cbar : (optional) bool
         Whether or not to show a colorbar (instead of a legend)
+
     """
     if not isinstance(axis, mpl.axes.Axes):
         raise TypeError('axis must be a matplotlib.axes.Axes object')
@@ -474,7 +506,8 @@ def plot_line_family(axis, x_vec, line_family, line_names=None, label_prefix='',
     if not isinstance(line_family, np.ndarray):
         raise TypeError('line_family must be a 2d array of numbers')
     assert line_family.ndim == 2, 'line_family must be a 2D array'
-    assert x_vec.size == line_family.shape[1], 'The size of the 2nd dimension of line_family must match with of x_vec'
+    #    assert x_vec.shape[1] == line_family.shape[1], \
+    #        'The size of the 2nd dimension of line_family must match with of x_vec, but line fam has shape {} whereas xvec has shape {}'.format(line_family.shape, x_vec.shape)
     num_lines = line_family.shape[0]
     for var, var_name in zip([label_suffix, label_prefix], ['label_suffix', 'label_prefix']):
         if not isinstance(var, (str, unicode)):
@@ -510,10 +543,11 @@ def plot_line_family(axis, x_vec, line_family, line_names=None, label_prefix='',
 
 
 def plot_map(axis, img, show_xy_ticks=True, show_cbar=True, x_vec=None, y_vec=None,
-             num_ticks=4, stdevs=None, cbar_label=None, tick_font_size=14, **kwargs):
+             num_ticks=4, stdevs=None, cbar_label=None, tick_font_size=None, infer_aspect=False, **kwargs):
     """
     Plots an image within the given axis with a color bar + label and appropriate X, Y tick labels.
     This is particularly useful to get readily interpretable plots for papers
+
     Parameters
     ----------
     axis : matplotlib.axes.Axes object
@@ -525,7 +559,7 @@ def plot_map(axis, img, show_xy_ticks=True, show_cbar=True, x_vec=None, y_vec=No
     show_cbar : bool, optional, default = True
         Whether or not to show the colorbar
     x_vec : 1-D array-like or Number, optional
-        if an array-like is provided - these will be used for the tick values on the X axis
+        if an array-like is provided, these will be used for the tick values on the X axis
         if a Number is provided, this will serve as an extent for tick values in the X axis.
         For example x_vec=1.5 would cause the x tick labels to range from 0 to 1.5
     y_vec : 1-D array-like or Number, optional
@@ -538,10 +572,13 @@ def plot_map(axis, img, show_xy_ticks=True, show_cbar=True, x_vec=None, y_vec=No
         Number of standard deviations to consider for plotting.  If None, full range is plotted.
     cbar_label : str, optional, default = None
         Labels for the colorbar. Use this for something like quantity (units)
-    tick_font_size : unsigned int, optional, default = 14
+    tick_font_size : unsigned int, optional, default = None
         Font size to apply to x, y, colorbar ticks and colorbar label
+    infer_aspect : bool, Optional. Default = False
+        Whether or not to adjust the aspect ratio of the image based on the provided x_vec and y_vec
+        The values of x_vec and y_vec will be assumed to have the same units.
     kwargs : dictionary
-        Anything else that will be passed on to imshow
+        Anything else that will be passed on to matplotlib.pyplot.imshow
 
     Returns
     -------
@@ -553,6 +590,7 @@ def plot_map(axis, img, show_xy_ticks=True, show_cbar=True, x_vec=None, y_vec=No
     Note
     ----
     The origin of the image will be set to the lower left corner. Use the kwarg 'origin' to change this
+
     """
     if not isinstance(axis, mpl.axes.Axes):
         raise TypeError('axis must be a matplotlib.axes.Axes object')
@@ -580,60 +618,94 @@ def plot_map(axis, img, show_xy_ticks=True, show_cbar=True, x_vec=None, y_vec=No
             raise TypeError('stdevs should be a Number')
         data_mean = np.mean(img)
         data_std = np.std(img)
-        kwargs.update({'clim': [data_mean - stdevs * data_std,
-                                data_mean + stdevs * data_std]})
+        kwargs.update({'clim': [data_mean - stdevs * data_std, data_mean + stdevs * data_std]})
 
     kwargs.update({'origin': kwargs.pop('origin', 'lower')})
 
-    im_handle = axis.imshow(img, **kwargs)
+    if show_cbar:
+        y_exp = get_exponent(np.squeeze(img))
+        z_suffix = ''
+        if y_exp < -2 or y_exp > 3:
+            img = np.squeeze(img) / 10 ** y_exp
+            z_suffix = ' x $10^{' + str(y_exp) + '}$'
+
     assert isinstance(show_xy_ticks, bool)
 
-    if show_xy_ticks is True or x_vec is not None:
+    ########################################################################################################
 
-        x_ticks = np.linspace(0, img.shape[1] - 1, num_ticks, dtype=int)
-        if x_vec is not None:
-            if isinstance(x_vec, (int, float)):
-                if x_vec > 0.01:
-                    x_tick_labs = [str(np.round(ind * x_vec / img.shape[1], 2)) for ind in x_ticks]
-                else:
-                    x_tick_labs = ['{0:.2e}'.format(ind * x_vec / img.shape[1]) for ind in x_ticks]
-            else:
-                if not isinstance(x_vec, (np.ndarray, list, tuple, range)) or len(x_vec) != img.shape[1]:
-                    raise ValueError(
-                        'x_vec should be array-like with shape equal to the second axis of img or img_size')
-                x_tick_labs = [str(np.round(x_vec[ind], 2)) for ind in x_ticks]
+    def set_ticks_for_axis(tick_vals, is_x):
+        if is_x:
+            tick_vals_var_name = 'x_vec'
+            tick_set_func = axis.set_xticks
+            tick_labs_set_func = axis.set_xticklabels
         else:
-            x_tick_labs = [str(ind) for ind in x_ticks]
+            tick_vals_var_name = 'y_vec'
+            tick_set_func = axis.set_yticks
+            tick_labs_set_func = axis.set_yticklabels
 
-        axis.set_xticks(x_ticks)
-        axis.set_xticklabels(x_tick_labs)
+        img_axis = int(is_x)
+        img_size = img.shape[img_axis]
+        chosen_ticks = np.linspace(0, img_size - 1, num_ticks, dtype=int)
 
-        set_tick_font_size(axis, tick_font_size)
+        if tick_vals is not None:
+            if isinstance(tick_vals, (int, float)):
+                if tick_vals > 0.01:
+                    tick_labs = [str(np.round(ind * tick_vals / img_size, 2)) for ind in chosen_ticks]
+                else:
+                    tick_labs = ['{0:.1E}'.format(ind * tick_vals / img_size) for ind in chosen_ticks]
+                    print(tick_labs)
+                tick_vals = np.linspace(0, tick_vals, img_size)
+            else:
+                if not isinstance(tick_vals, (np.ndarray, list, tuple, range)) or len(tick_vals) != img_size:
+                    raise ValueError(
+                        '{} should be array-like with shape equal to axis {} of img'.format(tick_vals_var_name,
+                                                                                            img_axis))
+                if np.max(tick_vals) > 0.01:
+                    tick_labs = [str(np.round(tick_vals[ind], 2)) for ind in chosen_ticks]
+                else:
+                    tick_labs = ['{0:.1E}'.format(tick_vals[ind]) for ind in chosen_ticks]
+        else:
+            tick_labs = [str(ind) for ind in chosen_ticks]
+
+        tick_set_func(chosen_ticks)
+        tick_labs_set_func(tick_labs)
+
+        if tick_font_size is not None:
+            set_tick_font_size(axis, tick_font_size)
+
+        return tick_vals
+
+    ########################################################################################################
+
+    if show_xy_ticks is True or x_vec is not None:
+        x_vec = set_ticks_for_axis(x_vec, True)
     else:
         axis.set_xticks([])
 
     if show_xy_ticks is True or y_vec is not None:
-        y_ticks = np.linspace(0, img.shape[0] - 1, num_ticks, dtype=int)
-        if y_vec is not None:
-            if isinstance(y_vec, (int, float)):
-                if y_vec > 0.01:
-                    y_tick_labs = [str(np.round(ind * y_vec / img.shape[1], 2)) for ind in y_ticks]
-                else:
-                    y_tick_labs = ['{0:.2e}'.format(ind * y_vec / img.shape[1]) for ind in y_ticks]
-            else:
-                if not isinstance(y_vec, (np.ndarray, list, tuple, range)) or len(y_vec) != img.shape[0]:
-                    raise ValueError('y_vec should be array-like with shape equal to the first axis of img')
-                y_tick_labs = [str(np.round(y_vec[ind], 2)) for ind in y_ticks]
-        else:
-            y_tick_labs = [str(ind) for ind in y_ticks]
-
-        axis.set_yticks(y_ticks)
-        axis.set_yticklabels(y_tick_labs)
-
-        set_tick_font_size(axis, tick_font_size)
+        y_vec = set_ticks_for_axis(y_vec, False)
     else:
         axis.set_yticks([])
 
+    if infer_aspect:
+        # Aspect ratio determined by this function will take precedence.
+        _ = kwargs.pop('infer_aspect', None)
+
+        """
+        At this stage, if x_vec and y_vec are not None, they should be arrays.
+        
+        This will be very useful when one dimension is coarsely sampled while another is finely sampled
+        and we want to visualize the image with the physically correct aspect ratio.
+        This CANNOT be performed automatically due to potentially incompatible units which are unknown to this func.
+        """
+
+        if x_vec is not None or y_vec is not None:
+            x_range = x_vec.max() - x_vec.min()
+            y_range = y_vec.max() - y_vec.min()
+            kwargs.update({'aspect': (y_range / x_range) * (img.shape[1] / img.shape[0])})
+
+    im_handle = axis.imshow(img, **kwargs)
+
     cbar = None
     if not isinstance(show_cbar, bool):
         show_cbar = False
@@ -646,8 +718,17 @@ def plot_map(axis, img, show_xy_ticks=True, show_cbar=True, x_vec=None, y_vec=No
         if cbar_label is not None:
             if not isinstance(cbar_label, (str, unicode)):
                 raise TypeError('cbar_label should be a string')
-            cbar.set_label(cbar_label, fontsize=tick_font_size)
-        cbar.ax.tick_params(labelsize=tick_font_size)
+
+            if tick_font_size is not None:
+                cbar.set_label(cbar_label + z_suffix)
+            else:
+                cbar.set_label(cbar_label + z_suffix, fontsize=tick_font_size)
+        else:
+            if z_suffix is not '':
+                cbar.set_label(z_suffix)
+
+        if tick_font_size is not None:
+            cbar.ax.tick_params(labelsize=tick_font_size)
     return im_handle, cbar
 
 
@@ -687,6 +768,7 @@ def plot_curves(excit_wfms, datasets, line_colors=[], dataset_names=[], evenly_s
     Returns
     ---------
     fig, axes
+
     """
     for var, var_name in zip([use_rainbow_plots, evenly_spaced], ['use_rainbow_plots', 'evenly_spaced']):
         if not isinstance(var, bool):
